function [pdec,sepdec]=decomposition_cbcrep(y,x,xm,nc,beta1,H1,N1,N2,beta2,H2,R,epsilon,family)

%Estimator of the decomposition of the difference in probability of a woman
%being employed conditional on the husband working vs not working, into an
%endowment effect and a homophily effect; CBRE with parametric copula

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%beta1=cbre parameters
%H1=cbre estimator covariance matrix
%N1,N2=number of points for copula grid
%beta2=re parameters
%H2=re estimator covariance matrix
%R=number of points for the Gauss-Hermite quadrature
%epsilon={logit,probit}
%family={Gaussian,Clayton,Frank,Gumbel,Ali,Joe}

%output:
%pdec=estimated probabilities
%sepdec=standard errors

[~,T]=size(y);%number of periods
[~,K]=size(x);
K=K/T;%number of covariates
[~,Kv]=size(xm);
cumnc=[0;cumsum(nc)];
C=length(nc);%number of clusters

grid=gridcopula(family,beta1(end),N1,N2);%grid of individual effects with dependence
eta=norminv(grid);

%Classify wives into married to employed and unemployed husbands
he=zeros(C,T);
for i1=1:1:C
    for i2=1:1:T
        he(i1,i2)=y(2*(i1-1)+1,i2);
    end
end
Nhe=sum(sum(he));
Nhu=C*T-Nhe;

%Compute basic probabilities with the CBRE: p(h,w),p(h),p(w|h)
prob1=zeros(C,T);
prob2=zeros(C,T);
prob3=zeros(C,T);
for i1=1:1:C
    xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)',kron(xm(cumnc(i1)+1:cumnc(i1+1),:),ones(T,1))];
    for i2=1:1:N1
        z=kron(beta1(end-1)*eta(i2,:)',ones(1,2*T))+ones(N2,1)*beta1(1:T+K+Kv)'*xi';
        for i3=1:1:T
            if he(i1,i3)==1
                switch epsilon
                    case 'logit'
                        prob1(i1,i3)=prob1(i1,i3)+mean(exp(z(:,i3))./(1+exp(z(:,i3)))).*mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                        prob2(i1,i3)=prob2(i1,i3)+mean(exp(z(:,i3))./(1+exp(z(:,i3))))/N1;
                        prob3(i1,i3)=prob3(i1,i3)+mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                    case 'probit'
                        prob1(i1,i3)=prob1(i1,i3)+mean(normcdf(z(:,i3))).*mean(normcdf(z(:,T+i3)))/N1;
                        prob2(i1,i3)=prob2(i1,i3)+mean(normcdf(z(:,i3)))/N1;
                        prob3(i1,i3)=prob3(i1,i3)+mean(normcdf(z(:,T+i3)))/N1;
                end
            elseif he(i1,i3)==0
                switch epsilon
                    case 'logit'
                        prob1(i1,i3)=prob1(i1,i3)+mean(1./(1+exp(z(:,i3)))).*mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                        prob2(i1,i3)=prob2(i1,i3)+mean(1./(1+exp(z(:,i3))))/N1;
                        prob3(i1,i3)=prob3(i1,i3)+mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                    case 'probit'
                        prob1(i1,i3)=prob1(i1,i3)+mean(1-normcdf(z(:,i3))).*mean((normcdf(z(:,T+i3))))/N1;
                        prob2(i1,i3)=prob2(i1,i3)+mean(1-normcdf(z(:,i3)))/N1;
                        prob3(i1,i3)=prob3(i1,i3)+mean(normcdf(z(:,T+i3)))/N1;
                end
            end
        end
    end
end
prob4=prob1./prob2;

%Compute the basic probability with the RE: p(w)=p(w|h)
[u,wu]=GaussHermite_2(R);

prob5=zeros(C,T);
for i1=1:1:C
    xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)',kron(xm(cumnc(i1)+1:cumnc(i1+1),:),ones(T,1))];
    z=kron(u*beta2(T+K+Kv+1)*sqrt(2),ones(1,2*T))+ones(R,1)*beta2(1:T+K+Kv)'*xi';
    for i2=1:1:T
        switch epsilon
            case 'logit'
                prob5(i1,i2)=wu'*(exp(z(:,T+i2))./(1+exp(z(:,T+i2))))/sqrt(pi);
            case 'probit'
                prob5(i1,i2)=wu'*(normcdf(z(:,T+i2)))/sqrt(pi);
        end
    end
end

pdec=zeros(9,1);
pdec(1)=sum(sum(prob3))/(C*T);%P(w=1);CBRE
pdec(2)=sum(sum(prob5))/(C*T);%P(w=1);RE
pdec(3)=sum(sum(prob4.*he))/Nhe;%P(w=1|h=1);CBRE
pdec(4)=sum(sum(prob4.*(1-he)))/Nhu;%P(w=1|h=0);CBRE
pdec(5)=sum(sum(prob5.*he))/Nhe;%P(w=1|h=1);RE
pdec(6)=sum(sum(prob5.*(1-he)))/Nhu;%P(w=1|h=0);RE
pdec(7)=pdec(3)-pdec(4);%CBRE dif
pdec(8)=pdec(5)-pdec(6);%RE dif=endowment effect
pdec(9)=pdec(7)-pdec(8);%homophily effect

%Numerically approximate the derivatives
eps=0.01;

dpdec=zeros(7,length(beta1)+length(beta2));
for i0=1:1:length(beta1)
    beta1(i0)=beta1(i0)+eps;
    prob1=zeros(C,T);
    prob2=zeros(C,T);
    prob3=zeros(C,T);
    for i1=1:1:C
        xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)',kron(xm(cumnc(i1)+1:cumnc(i1+1),:),ones(T,1))];
        for i2=1:1:N1
            z=kron(beta1(end-1)*eta(i2,:)',ones(1,2*T))+ones(N2,1)*beta1(1:T+K+Kv)'*xi';
            for i3=1:1:T
                if he(i1,i3)==1
                    switch epsilon
                        case 'logit'
                            prob1(i1,i3)=prob1(i1,i3)+mean(exp(z(:,i3))./(1+exp(z(:,i3)))).*mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                            prob2(i1,i3)=prob2(i1,i3)+mean(exp(z(:,i3))./(1+exp(z(:,i3))))/N1;
                            prob3(i1,i3)=prob3(i1,i3)+mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                        case 'probit'
                            prob1(i1,i3)=prob1(i1,i3)+mean(normcdf(z(:,i3))).*mean(normcdf(z(:,T+i3)))/N1;
                            prob2(i1,i3)=prob2(i1,i3)+mean(normcdf(z(:,i3)))/N1;
                            prob3(i1,i3)=prob3(i1,i3)+mean(normcdf(z(:,T+i3)))/N1;
                    end
                elseif he(i1,i3)==0
                    switch epsilon
                        case 'logit'
                            prob1(i1,i3)=prob1(i1,i3)+mean(1./(1+exp(z(:,i3)))).*mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                            prob2(i1,i3)=prob2(i1,i3)+mean(1./(1+exp(z(:,i3))))/N1;
                            prob3(i1,i3)=prob3(i1,i3)+mean(exp(z(:,T+i3))./(1+exp(z(:,T+i3))))/N1;
                        case 'probit'
                            prob1(i1,i3)=prob1(i1,i3)+mean(1-normcdf(z(:,i3))).*mean(normcdf(z(:,T+i3)))/N1;
                            prob2(i1,i3)=prob2(i1,i3)+mean(1-normcdf(z(:,i3)))/N1;
                            prob3(i1,i3)=prob3(i1,i3)+mean(normcdf(z(:,T+i3)))/N1;
                    end
                end
            end
        end
    end
    prob4=prob1./prob2;
    dpdec(1,i0)=(sum(sum(prob3))/(C*T)-pdec(1))/eps;
    dpdec(3,i0)=(sum(sum(prob4.*he))/Nhe-pdec(3))/eps;
    dpdec(4,i0)=(sum(sum(prob4.*(1-he)))/Nhu-pdec(4))/eps;
    dpdec(7,i0)=dpdec(3,i0)-dpdec(4,i0);
    dpdec(9,i0)=dpdec(7,i0);
    beta1(i0)=beta1(i0)-eps;
end

for i0=1:1:length(beta2)
    beta2(i0)=beta2(i0)+eps;
    prob5=zeros(C,T);
    for i1=1:1:C
        xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)',kron(xm(cumnc(i1)+1:cumnc(i1+1),:),ones(T,1))];
        z=kron(u*beta2(T+K+Kv+1)*sqrt(2),ones(1,2*T))+ones(R,1)*beta2(1:T+K+Kv)'*xi';
        for i2=1:1:T
            switch epsilon
                case 'logit'
                    prob5(i1,i2)=wu'*(exp(z(:,T+i2))./(1+exp(z(:,T+i2))))/sqrt(pi);
                case 'probit'
                    prob5(i1,i2)=wu'*(normcdf(z(:,T+i2)))/sqrt(pi);
            end
        end
    end
    dpdec(2,length(beta1)+i0)=(sum(sum(prob5))/(C*T)-pdec(2))/eps;
    dpdec(5,length(beta1)+i0)=(sum(sum(prob5.*he))/Nhe-pdec(5))/eps;
    dpdec(6,length(beta1)+i0)=(sum(sum(prob5.*(1-he)))/Nhu-pdec(6))/eps;
    dpdec(8,length(beta1)+i0)=dpdec(5,length(beta1)+i0)-dpdec(6,length(beta1)+i0);
    dpdec(9,length(beta1)+i0)=-dpdec(8,length(beta1)+i0);
    beta2(i0)=beta2(i0)-eps;
end

%compute standard errors
Vpdec=dpdec*[H1,zeros(length(beta1),length(beta2));zeros(length(beta2),length(beta1)),H2]*dpdec';
sepdec=sqrt(diag(Vpdec));