function [pwork,sepwork,dpwork,jointp,djointp]=probabilitywork_cbreb(y,x,nc,b,a,H,epsilon,N1,d)

%Estimator of the joint probability for the CBRE estimator with Bernstein
%copula
%estimated probabilities:
%P(at least one works in every period with dependence)
%P(at least one works in every period with independence)
%difference in probabilities

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%b=marginal parameters
%a=Bernstein copula parameters
%H=covariance matrix
%epsilon={logit,probit}
%N1=number of points for copula grid
%d=order of Bernstein copula

%output:
%pwork=estimated average probabilities
%sepwork=standard errors
%dpwork=derivative of pwork wrt to parameters (ie bread of sandwich
%formula)
%jointp=estimated individual (ie by cluster) probabilities
%djointp=derivative of joinp wrt to parameters (ie bread of sandwich
%formula)

[~,T]=size(y);%number of periods
[~,K]=size(x);
K=K/T;%number of covariates
cumnc=[0;cumsum(nc)];
C=length(nc);%number of clusters

grid=linspace(1/(N1+1),N1/(N1+1),N1);%grid of individual effects
eta=norminv(grid);

A=zeros(d+1,d+1);%alpha parameters as in Sancetta, Satchell
A(end,:)=linspace(0,1,d+1);
A(:,end)=A(end,:)';
A(2:end-1,2:end-1)=reshape(a,d-1,d-1);
B=(A(2:end,2:end)+A(1:end-1,1:end-1)-A(2:end,1:end-1)-A(1:end-1,2:end))*d^2;%beta parameters as in Sancetta, Satchell
binoms=zeros(1,d);
for i1=0:1:d-1
    binoms(i1+1)=nchoosek(d-1,i1);
end
polord=0:1:d-1;
P=(ones(N1,1)*binoms).*(grid'.^(polord)).*(1-grid').^(d-1-polord);
c=P*B*P';%density of Bernstein copula
c0=ones(N1,N1);%density of independence copula

%Compute the probability of interest for each individual, then take the
%mean
jointp=zeros(3,C);
for i1=1:1:C
    xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)'];
    z=xi*b(1:T+K)+b(T+K+1)*eta;
    %Compute the probability of interest conditional on each value of the
    %individual effects,then integrate over its distribution
    switch epsilon
        case 'logit'
            z2=exp(z)./(1+exp(z));
        case 'probit'
            z2=normcdf(z);
    end
    paux=zeros(1,T);
    paux0=zeros(1,T);
    for i2=1:1:T
        paux(1,i2)=mean(z2(i2,:))+mean(z2(T+i2,:))-z2(i2,:)*c*z2(T+i2,:)'/sum(sum(c));
        paux0(1,i2)=mean(z2(i2,:))+mean(z2(T+i2,:))-z2(i2,:)*c0*z2(T+i2,:)'/sum(sum(c0));
    end
    jointp(1,i1)=mean(prod(paux,2));
    jointp(2,i1)=mean(prod(paux0,2));
    jointp(3,i1)=jointp(1,i1)-jointp(2,i1);
end

%numerically approximate the derivatives
eps=0.001;

bb=[b;a];
djointp=zeros(length(bb),C,3);
jointpaux=zeros(length(bb),C);
jointpaux0=zeros(length(bb),C);
for i0=1:1:length(bb)
    bb(i0)=bb(i0)+eps;
    b=bb(1:length(b));
    a=bb(length(b)+1:end);
    
    A=zeros(d+1,d+1);
    A(end,:)=linspace(0,1,d+1);
    A(:,end)=A(end,:)';
    A(2:end-1,2:end-1)=reshape(a,d-1,d-1);
    B=(A(2:end,2:end)+A(1:end-1,1:end-1)-A(2:end,1:end-1)-A(1:end-1,2:end))*d^2;
    binoms=zeros(1,d);
    for i1=0:1:d-1
        binoms(i1+1)=nchoosek(d-1,i1);
    end
    polord=0:1:d-1;
    P=(ones(N1,1)*binoms).*(grid'.^(polord)).*(1-grid').^(d-1-polord);
    c=P*B*P';
    for i1=1:1:C
        xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)'];
        z=xi*b(1:T+K)+b(T+K+1)*eta;
        %Compute the probability of interest conditional on each value of the
        %individual effects,then integrate over its distribution
        switch epsilon
            case 'logit'
                z2=exp(z)./(1+exp(z));
            case 'probit'
                z2=normcdf(z);
        end
        paux=zeros(1,T);
        paux0=zeros(1,T);
        for i2=1:1:T
            paux(1,i2)=mean(z2(i2,:))+mean(z2(T+i2,:))-z2(i2,:)*c*z2(T+i2,:)'/sum(sum(c));
            paux0(1,i2)=mean(z2(i2,:))+mean(z2(T+i2,:))-z2(i2,:)*c0*z2(T+i2,:)'/sum(sum(c0));
        end
        jointpaux(i0,i1)=mean(prod(paux,2));
        jointpaux0(i0,i1)=mean(prod(paux0,2));
    end
    bb(i0)=bb(i0)-eps;
end
for i1=1:1:length(bb)
    djointp(i1,:,1)=(jointpaux(i1,:)-jointp(1,:))/eps;
    djointp(i1,:,2)=(jointpaux0(i1,:)-jointp(2,:))/eps;
    djointp(i1,:,3)=djointp(i1,:,1)-djointp(i1,:,2);
end

pwork=mean(jointp,2);
dpwork=reshape(mean(djointp,2),length(bb),3);
sepwork=[sqrt(dpwork(:,1)'*H*dpwork(:,1));
    sqrt(dpwork(:,2)'*H*dpwork(:,2));
    sqrt(dpwork(:,3)'*H*dpwork(:,3))];