function [pwork,sepwork,dpwork,jointp,djointp]=probabilitywork_cbrep(y,x,nc,b,H,epsilon,family,N1,N2)

%Estimator of the joint probability for the CBRE estimator
%estimated probabilities:
%P(at least one works in every period with dependence)
%P(at least one works in every period with independence)
%difference in probabilities

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%b=parameters
%H=covariance matrix
%epsilon={logit,probit}
%family={Gaussian,Clayton,Frank,Gumbel,Ali,Joe}
%N1,N2=number of points for copula grid

%output:
%pwork=estimated average probabilities
%sepwork=standard errors
%dpwork=derivative of pwork wrt to parameters (ie bread of sandwich
%formula)
%jointp=estimated individual (ie by cluster) probabilities
%djointp=derivative of joinp wrt to parameters (ie bread of sandwich
%formula)

[~,T]=size(y);%number of periods
[~,K]=size(x);
K=K/T;%number of covariates
cumnc=[0;cumsum(nc)];
C=length(nc);%number of clusters

grid=gridcopula(family,b(end),N1,N2);%grid of individual effects with dependence
eta=norminv(grid);
grid0=ones(N1,1)*linspace(1/(N2+1),N2/(N2+1),N2);%grid of individual effects with independence
eta0=norminv(grid0);

%Compute the probability of interest for each individual, then take the
%mean
jointp=zeros(3,C);
for i1=1:1:C
    xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)'];
    paux=zeros(N1,T);
    paux0=zeros(N1,T);
    %Compute the probability of interest conditional on each value of the
    %individual effects,then integrate over its distribution
    for i2=1:1:N1
        z=kron(b(end-1)*eta(i2,:)',ones(1,2*T))+ones(N2,1)*b(1:T+K)'*xi';
        z0=kron(b(end-1)*eta0(i2,:)',ones(1,2*T))+ones(N2,1)*b(1:T+K)'*xi';
        h=zeros(1,2*T);
        h0=zeros(1,2*T);
        switch epsilon
            case 'logit'
                for i3=1:1:2*T
                    h(i3)=mean(1-min(max(1./(1+exp(z(:,i3))),10^-15),1-10^-15));
                    h0(i3)=mean(1-min(max(1./(1+exp(z0(:,i3))),10^-15),1-10^-15));
                end
            case 'probit'
                for i3=1:1:2*T
                    h(i3)=mean(min(max(normcdf(z(:,i3)),10^-15),1-10^-15));
                    h0(i3)=mean(min(max(normcdf(z0(:,i3)),10^-15),1-10^-15));
                end
        end
        for i3=1:1:T
            paux(i2,i3)=h(i3)+h(T+i3)-h(i3)*h(T+i3);
            paux0(i2,i3)=h0(i3)+h0(T+i3)-h0(i3)*h0(T+i3);
        end
    end
    jointp(1,i1)=mean(prod(paux,2));
    jointp(2,i1)=mean(prod(paux0,2));
    jointp(3,i1)=jointp(1,i1)-jointp(2,i1);
end

%numerically approximate the derivatives
eps=0.01;

djointp=zeros(length(b),C,3);
jointpaux=zeros(length(b),C);
jointpaux0=zeros(length(b),C);
for i0=1:1:length(b)
    b(i0)=b(i0)+eps;
    grid=gridcopula(family,b(end),N1,N2);
    eta=norminv(grid);
    grid0=ones(N1,1)*linspace(1/(N2+1),N2/(N2+1),N2);
    eta0=norminv(grid0);
    for i1=1:1:C
        xi=[kron(ones(nc(i1),1),eye(T)),reshape(x(cumnc(i1)+1:cumnc(i1+1),:)',K,nc(i1)*T)'];
        paux=zeros(N1,T);
        paux0=zeros(N1,T);
        for i2=1:1:N1
            z=kron(b(end-1)*eta(i2,:)',ones(1,2*T))+ones(N2,1)*b(1:T+K)'*xi';
            z0=kron(b(end-1)*eta0(i2,:)',ones(1,2*T))+ones(N2,1)*b(1:T+K)'*xi';
            h=zeros(1,2*T);
            h0=zeros(1,2*T);
            switch epsilon
                case 'logit'
                    for i3=1:1:2*T
                        h(i3)=mean(1-min(max(1./(1+exp(z(:,i3))),10^-15),1-10^-15));
                        h0(i3)=mean(1-min(max(1./(1+exp(z0(:,i3))),10^-15),1-10^-15));
                    end
                case 'probit'
                    for i3=1:1:2*T
                        h(i3)=mean(min(max(normcdf(z(:,i3)),10^-15),1-10^-15));
                        h0(i3)=mean(min(max(normcdf(z0(:,i3)),10^-15),1-10^-15));
                    end
            end
            for i3=1:1:T
                paux(i2,i3)=h(i3)+h(T+i3)-h(i3)*h(T+i3);
                paux0(i2,i3)=h0(i3)+h0(T+i3)-h0(i3)*h0(T+i3);
            end
        end
        jointpaux(i0,i1)=mean(prod(paux,2));
        jointpaux0(i0,i1)=mean(prod(paux0,2));
    end
    b(i0)=b(i0)-eps;
end
for i1=1:1:length(b)
    djointp(i1,:,1)=(jointpaux(i1,:)-jointp(1,:))/eps;
    djointp(i1,:,2)=(jointpaux0(i1,:)-jointp(2,:))/eps;
    djointp(i1,:,3)=djointp(i1,:,1)-djointp(i1,:,2);
end

pwork=mean(jointp,2);
dpwork=reshape(mean(djointp,2),length(b),3);
sepwork=[sqrt(dpwork(:,1)'*H*dpwork(:,1));
    sqrt(dpwork(:,2)'*H*dpwork(:,2));
    sqrt(dpwork(:,3)'*H*dpwork(:,3))];