function [jointp,djointp,condp,dcondp]=jointprobs(y,x,nc,b,family,epsilon,N1,N2)

%Estimator of the joint and conditional probabilities for the CBRE
%estimator
%The probability is calculated for (y1,y2) in each group and period

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%b=initial value of parameters
%family={Gaussian,Clayton,Frank,Gumbel}
%epsilon={logit,probit}
%N1,N2=number of points for copula grid

%output:
%jointp=joint probabilities:(1,1),(1,0),(0,1),(0,0)
%djointp=derivatives of the joint probabilities wrt parameters
%condp=conditional probabilities:(1|1),(1|0)
%dcondp=derivatives of the conditional probabilities wrt parameters

[~,T]=size(y);%number of periods
[~,K]=size(x);
K=K/T;%number of covariates
cumnc=[0;cumsum(nc)];
C=length(nc);%number of groups

%predefine some variables
jointp=zeros(C,4*T);
condp=zeros(C,2*T);
for i1=1:1:C
    grid=gridcopula(family,b(end),N1,N2);%grid for the individual effects
    eta=b(end-1)*norminv(grid);
    xi=[kron(ones(nc(i1),1),eye(T)),zeros(nc(i1)*T,K)];%covariates
    for i2=1:1:nc(i1)
        xi(T*(i2-1)+1:T*i2,T+1:end)=reshape(x(cumnc(i1)+i2,:),K,T)';
    end
    z=exp(xi*b(1:end-2));
    
    %compute joint probabilities
    for i2=1:1:N1
        switch epsilon
            case 'logit'
                p1=z*exp(eta(i2,:))./(1+z*exp(eta(i2,:)));%logit cdf
            case 'probit'
                p1=normcdf(eta(i2,:)+log(z));%normal cdf
        end
        
        p2=zeros(T,1);
        p3=zeros(T,1);
        p4=zeros(T,1);
        p5=zeros(T,1);
        for i3=1:1:T
            p2(i3,1)=mean(p1(i3,:))*mean(p1(T+i3,:));
            p3(i3,1)=(1-mean(p1(i3,:)))*mean(p1(T+i3,:));
            p4(i3,1)=mean(p1(i3,:))*(1-mean(p1(T+i3,:)));
            p5(i3,1)=(1-mean(p1(i3,:)))*(1-mean(p1(T+i3,:)));
            jointp(i1,(i3-1)*4+1)=jointp(i1,(i3-1)*4+1)+p2(i3,1)/N1;%(1,1)
            jointp(i1,(i3-1)*4+2)=jointp(i1,(i3-1)*4+2)+p3(i3,1)/N1;%(0,1)
            jointp(i1,(i3-1)*4+3)=jointp(i1,(i3-1)*4+3)+p4(i3,1)/N1;%(1,0)
            jointp(i1,(i3-1)*4+4)=jointp(i1,(i3-1)*4+4)+p5(i3,1)/N1;%(0,0)
        end
    end
    
    %compute conditional probabilities
    for i2=1:1:T
        condp(:,(i2-1)*2+1)=jointp(:,(i2-1)*4+1)./(jointp(:,(i2-1)*4+1)+jointp(:,(i2-1)*4+2));%(1|1)
        condp(:,(i2-1)*2+2)=jointp(:,(i2-1)*4+3)./(jointp(:,(i2-1)*4+3)+jointp(:,(i2-1)*4+4));%(1|0)
    end
end

%approximate derivatives numerically
eps=0.01;
djointp=zeros(C,length(b),4*T);
dcondp=zeros(C,length(b),2*T);
jointpaux=zeros(C,length(b),4*T);
condpaux=zeros(C,length(b),2*T);
for i0=1:1:length(b)
    b(i0)=b(i0)+eps;
    for i1=1:1:C
        grid=gridcopula(family,b(end),N1,N2);
        eta=b(end-1)*norminv(grid);
        xi=[kron(ones(nc(i1),1),eye(T)),zeros(nc(i1)*T,K)];
        for i2=1:1:nc(i1)
            xi(T*(i2-1)+1:T*i2,T+1:end)=reshape(x(cumnc(i1)+i2,:),K,T)';
        end
        z=exp(xi*b(1:end-2));
        
        for i2=1:1:N1
            switch epsilon
                case 'logit'
                    p1=z*exp(eta(i2,:))./(1+z*exp(eta(i2,:)));
                case 'probit'
                    p1=normcdf(eta(i2,:)+z);
            end
            p2=zeros(T,1);
            p3=zeros(T,1);
            p4=zeros(T,1);
            p5=zeros(T,1);
            for i3=1:1:T
                p2(i3,1)=mean(p1(i3,:))*mean(p1(T+i3,:));
                p3(i3,1)=(1-mean(p1(i3,:)))*mean(p1(T+i3,:));
                p4(i3,1)=mean(p1(i3,:))*(1-mean(p1(T+i3,:)));
                p5(i3,1)=(1-mean(p1(i3,:)))*(1-mean(p1(T+i3,:)));
                jointpaux(i1,i0,(i3-1)*4+1)=jointpaux(i1,i0,(i3-1)*4+1)+p2(i3,1)/N1;
                jointpaux(i1,i0,(i3-1)*4+2)=jointpaux(i1,i0,(i3-1)*4+2)+p3(i3,1)/N1;
                jointpaux(i1,i0,(i3-1)*4+3)=jointpaux(i1,i0,(i3-1)*4+3)+p4(i3,1)/N1;
                jointpaux(i1,i0,(i3-1)*4+4)=jointpaux(i1,i0,(i3-1)*4+4)+p5(i3,1)/N1;
            end
        end
    end
    for i2=1:1:T
        condpaux(:,i0,(i2-1)*2+1)=jointpaux(:,i0,(i2-1)*4+1)./(jointpaux(:,i0,(i2-1)*4+1)+jointpaux(:,i0,(i2-1)*4+2));
        condpaux(:,i0,(i2-1)*2+2)=jointpaux(:,i0,(i2-1)*4+3)./(jointpaux(:,i0,(i2-1)*4+3)+jointpaux(:,i0,(i2-1)*4+4));
    end
    b(i0)=b(i0)-eps;
end
for i1=1:1:size(condpaux,3)
    djointp(:,:,i1)=(log(jointpaux(:,:,i1))-log(jointp(:,i1))*ones(1,length(b)))/eps;
    dcondp(:,:,i1)=(log(condpaux(:,:,i1))-log(condp(:,i1))*ones(1,length(b)))/eps;
end